﻿// ReSharper disable once RedundantUsingDirective
using System;
using System.Linq;
using System.Threading.Tasks;
using Windows.ApplicationModel.Activation;
using Windows.ApplicationModel.Core;
using Windows.Storage;
using Windows.Storage.Pickers;

namespace UniversalSample.Commons
{
    /// <summary>
    /// StorageHelper
    /// </summary>
    public class StorageHelper
    {
        private static StorageHelper _instance;

        /// <summary>
        /// Instance
        /// </summary>
        public static StorageHelper Instance
        {
            get { return _instance = _instance ?? new StorageHelper(); }
        }

#if WINDOWS_APP
        public async Task<StorageFile> PickSingleFileAsync(FileOpenPicker picker) 
        {
            StorageFile file = await picker.PickSingleFileAsync();
            return file;
        }
#else
        //Windows Phone 8.1

        /// <summary>
        /// completionSource
        /// </summary>
        private TaskCompletionSource<StorageFile> completionSource;

        /// <summary>
        /// PickSingleFileAsync
        /// </summary>
        /// <param name="picker"></param>
        /// <returns></returns>
        public async Task<StorageFile> PickSingleFileAsync(FileOpenPicker picker)
        {
            //TaskCompletionSource 생성
            completionSource = new TaskCompletionSource<StorageFile>();

            //앱의 Activated 이벤트 연결
            CoreApplication.GetCurrentView().Activated += OnApplicationActivated;

            //PickSingleFileAndContinue 메소드 실행
            picker.PickSingleFileAndContinue();

            //TaskCompletionSource가 완료될 때까지 기다림, 결과는 StorageFile
            var file = await completionSource.Task;

            //file 리턴
            return file;
        }

        /// <summary>
        /// OnApplicationActivated
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        private void OnApplicationActivated(CoreApplicationView sender, IActivatedEventArgs args)
        {
            //Args확인
            var continueArgs = args as FileOpenPickerContinuationEventArgs;
            
            if (continueArgs != null && continueArgs.Files.Any())
            {
                //completionSource가 존재하면
                if (completionSource != null)
                {
                    //결과 세팅
                    completionSource.TrySetResult(continueArgs.Files.First());
                    //completionSource 삭제
                    completionSource = null;
                }
            }
            else
            {
                //선택한 파일이 존재하지 않으면
                if (completionSource != null)
                {
                    //취소 세팅
                    completionSource.TrySetCanceled();
                    completionSource = null;
                }
            }
            //이벤트 연결 해제
            CoreApplication.GetCurrentView().Activated -= OnApplicationActivated;
        }
#endif
    }
}