﻿using System.Diagnostics;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace UniversalSample.Commons
{
    public class ViewModelBase : BindableBase
    {
        private RelayCommand _goBackCommand;
        private bool _isBusy;
        private RelayCommand _loadedCommand;
        private RelayCommand _sizeChangedCommand;

        private string _title;
        private RelayCommand _unloadedCommand;

        public ViewModelBase()
        {
            FrameHelper.Instance.Navigating += Instance_Navigating;
            FrameHelper.Instance.Navigated += Instance_Navigated;
        }

        private void Instance_Navigated(object sender, NavigationEventArgs e)
        {
            if (e == null || e.Content == null) return;
            var dataContext = ((FrameworkElement) e.Content).DataContext;
            if (dataContext != null && dataContext.Equals(this))
            {
                Debug.WriteLine(Title + " Navigated");
                GoBackCommand.RaiseCanExecuteChanged();
                OnNavigated(e);
            }
        }

        private void Instance_Navigating(object sender, NavigatingCancelEventArgs e)
        {
            if (!(sender is Frame)) return;
            var frame = (Frame)sender;
            var view = (FrameworkElement)frame.Content;
            if (view != null && view.DataContext.Equals(this))
            {
                Debug.WriteLine(Title + " Navigating");
                OnNavigating(e);
            }
        }

        /// <summary>
        ///     작업중
        /// </summary>
        public bool IsBusy
        {
            get { return _isBusy; }
            set
            {
                _isBusy = value;
                OnPropertyChanged();
            }
        }

        /// <summary>
        ///     제목
        /// </summary>
        public string Title
        {
            get { return _title; }
            set { SetProperty(ref _title, value); }
        }

        /// <summary>
        ///     로디드 커맨드
        /// </summary>
        public RelayCommand LoadedCommand
        {
            get { return _loadedCommand = _loadedCommand ?? new RelayCommand(OnLoaded); }
        }

        /// <summary>
        ///     언로드 커맨드
        /// </summary>
        public RelayCommand UnloadedCommand
        {
            get { return _unloadedCommand = _unloadedCommand ?? new RelayCommand(OnUnloaded); }
        }

        /// <summary>
        ///     사이즈 체인지 커맨드
        /// </summary>
        public RelayCommand SizeChangedCommand
        {
            get { return _sizeChangedCommand = _sizeChangedCommand ?? new RelayCommand(OnSizeChanged); }
        }

        /// <summary>
        ///     백 커맨드
        /// </summary>
        public RelayCommand GoBackCommand
        {
            get { return _goBackCommand = _goBackCommand ?? new RelayCommand(args => FrameHelper.Instance.GoBack(), () => FrameHelper.Instance.CanGoBack()); }
        }

        /// <summary>
        ///     뷰가 로드시에 실행되는 메소드
        /// </summary>
        protected virtual void OnLoaded(object args)
        {
        }

        /// <summary>
        ///     뷰가 언로드시에 실행되는 메소드
        /// </summary>
        protected virtual void OnUnloaded(object args)
        {
        }

        /// <summary>
        ///     사이즈 변경 이벤트 처리 메소드
        /// </summary>
        protected virtual void OnSizeChanged(object args)
        {
        }

        protected virtual void OnNavigating(NavigatingCancelEventArgs e) { }

        protected virtual void OnNavigated(NavigationEventArgs e) { }
    }
}