﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using UniversalSample.Commons;

namespace UniversalSample.Models
{
    /// <summary>
    /// MemoryItemModel, 메모리 아이템의 기본 모델
    /// </summary>
    public class MemoryItemM : BindableBase
    {
        private string _className;
        private IList<WeakReference> _weakReferences;

        /// <summary>
        ///     기본생성자
        /// </summary>
        public MemoryItemM()
        {
        }

        /// <summary>
        ///     생성자
        /// </summary>
        /// <param name="obj"></param>
        public MemoryItemM(object obj)
        {
            if (obj == null) return;
            ClassName = obj.GetType().FullName;
            AddObject(obj);
        }

        /// <summary>
        ///     클래스명
        /// </summary>
        public string ClassName
        {
            get { return _className; }
            private set { SetProperty(ref _className, value); }
        }

        /// <summary>
        ///     WeakReferences
        /// </summary>
        public IList<WeakReference> WeakReferences
        {
            get { return _weakReferences ?? (_weakReferences = new ObservableCollection<WeakReference>()); }
        }

        /// <summary>
        ///     AddObject
        /// </summary>
        /// <param name="obj"></param>
        public void AddObject(object obj)
        {
            if (string.IsNullOrEmpty(ClassName)
                || !ClassName.Equals(obj.GetType().FullName)) return;

            WeakReferences.Add(new WeakReference(obj));
        }

        /// <summary>
        ///     WeakReferences에 연결된 오브젝트 존재여부
        /// </summary>
        /// <returns></returns>
        public bool IsAliveWeakReferences()
        {
            List<WeakReference> deads = WeakReferences.Where(p => p.IsAlive.Equals(false)).ToList();
            if (deads.Any())
            {
                foreach (WeakReference dead in deads)
                {
                    WeakReferences.Remove(dead);
                }
            }
            return WeakReferences.Any();
        }

        /// <summary>
        ///     obj 존재여부 확인
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public bool IsExist(object obj)
        {
            return WeakReferences.FirstOrDefault(p => p.Target.Equals(obj)) != null;
        }
    }
}