﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using Windows.ApplicationModel;
using Windows.Foundation;
using Windows.Storage;
using Windows.Storage.FileProperties;
using Windows.Storage.Pickers;
using Windows.Storage.Streams;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;
using UniversalSample.Commons;
using UniversalSample.Design;
using UniversalSample.Models;
using UniversalSample.Views;

namespace UniversalSample.ViewModels
{
    public class MainPageVM : ViewModelBase
    {
        private RelayCommand _bitlyAuthCommand;
        private IList<ClassM> _classList;
        private RelayCommand _getShortenUrlCommand;
        private bool _isBitlyAuth;
        private RelayCommand _itemClickCommand;
        private string _longUrl;
        private RelayCommand _memoryViewCommand;
        private Popup _memoryViewPopup;
        private RelayCommand _openImageCommand;
        private RelayCommand _openPopupCommand;
        private IList<PersonM> _people;
        private ImageSource _selectedImageSource;
        private string _shortUrl;
        private RelayCommand _testCommand;
        private string _testText;

        public MainPageVM()
        {
            Title = "Kaki education center";

            ClassList = new List<ClassM>();
            People = new List<PersonM>();

            if (DesignMode.DesignModeEnabled)
            {
                //디자인 타임 데이터
                ClassList = DesignDatas.GetClassList();
                People = DesignDatas.GetPeople();
            }
            else
            {
                ViewModelLocator.MemoryViewer.AddObject(this);
                //LongUrl 프로퍼티 변경시 체크 커맨드
                PropertyChanged += (s, e) =>
                {
                    switch (e.PropertyName)
                    {
                        case "LongUrl":
                            CheckCommands();
                            break;
                    }
                };
            }
        }

        public IList<ClassM> ClassList
        {
            get { return _classList; }
            set
            {
                _classList = value;
                OnPropertyChanged();
            }
        }

        public IList<PersonM> People
        {
            get { return _people; }
            set
            {
                _people = value;
                OnPropertyChanged();
            }
        }

        /// <summary>
        ///     ItemClick Command
        /// </summary>
        public RelayCommand ItemClickCommand
        {
            get
            {
                return _itemClickCommand = _itemClickCommand ?? new RelayCommand(
                    args =>
                    {
                        if (!(args is ItemClickEventArgs)) return;

                        var item = ((ItemClickEventArgs) args).ClickedItem as ClassM;
                        if (item == null) return;

                        FrameHelper.Instance.Navigation(typeof (ClassDetailPage), item);
                    });
            }
        }

        public RelayCommand TestCommand
        {
            get
            {
                return _testCommand ??
                       (_testCommand =
                           new RelayCommand(obj => { FrameHelper.Instance.Navigation(typeof (BlankPage1), null); }));
            }
        }

        /// <summary>
        ///     Selected Bitmap Image
        /// </summary>
        public ImageSource SelectedImageSource
        {
            get { return _selectedImageSource; }
            set
            {
                SetProperty(ref _selectedImageSource, value);
            }
        }

        public string TestText
        {
            get { return _testText; }
            set
            {
                _testText = value;
                OnPropertyChanged();
            }
        }

        /// <summary>
        ///     Open Image Command
        /// </summary>
        public RelayCommand OpenImageCommand
        {
            get
            {
                return _openImageCommand = _openImageCommand ?? new RelayCommand(async args =>
                {
                    var openPicker = new FileOpenPicker
                    {
                        ViewMode = PickerViewMode.Thumbnail,
                        SuggestedStartLocation = PickerLocationId.PicturesLibrary
                    };
                    openPicker.FileTypeFilter.Add(".jpg");
                    openPicker.FileTypeFilter.Add(".jpeg");
                    openPicker.FileTypeFilter.Add(".png");
                    TestText = "FileOpen Start";
                    var file = await StorageHelper.Instance.PickSingleFileAsync(openPicker);
                    if (file != null)
                    {
                        var bitmapImage = new BitmapImage();
                        using (IRandomAccessStream fileStream = await file.OpenAsync(FileAccessMode.Read))
                        {
                            // Set the image source to the selected bitmap
                            ImageProperties property = await file.Properties.GetImagePropertiesAsync();
                            bitmapImage.DecodePixelHeight = (int) property.Height;
                            bitmapImage.DecodePixelWidth = (int) property.Width;
                            await bitmapImage.SetSourceAsync(fileStream);
                        }
                        SelectedImageSource = bitmapImage;
                        
                        ViewModelLocator.MemoryViewer.AddObject(bitmapImage);
                        TestText = "FileOpen End";
                    }
                });
            }
        }

        /// <summary>
        ///     Bitly Auth 여부
        /// </summary>
        public bool IsBitlyAuth
        {
            get { return _isBitlyAuth; }
            set { SetProperty(ref _isBitlyAuth, value); }
        }

        /// <summary>
        ///     Long Url
        /// </summary>
        public string LongUrl
        {
            get { return _longUrl; }
            set { SetProperty(ref _longUrl, value); }
        }

        /// <summary>
        ///     Shorten Url
        /// </summary>
        public string ShortUrl
        {
            get { return _shortUrl; }
            set { SetProperty(ref _shortUrl, value); }
        }

        /// <summary>
        ///     BitlyAuth Command
        /// </summary>
        public RelayCommand BitlyAuthCommand
        {
            get
            {
                return _bitlyAuthCommand = _bitlyAuthCommand ?? new RelayCommand(async args =>
                {
                    //아이디와 비밀번호는 자신의 것으로 사용한다. - 포스트 후에 비밀 번호 변경 할 예정임
                    IsBitlyAuth = await BitlyHelper.Instance.BitlyAuthAsync("kaki104", "zkzlroqkfxptmxm");
                    CheckCommands();
                }, () => !IsBitlyAuth);
            }
        }

        /// <summary>
        ///     GetShortenUrl Command
        /// </summary>
        public RelayCommand GetShortenUrlCommand
        {
            get
            {
                return
                    _getShortenUrlCommand =
                        _getShortenUrlCommand ??
                        new RelayCommand(
                            async args => { ShortUrl = await BitlyHelper.Instance.GetShortenUrl(LongUrl); },
                            () => IsBitlyAuth && string.IsNullOrEmpty(LongUrl) == false);
            }
        }

        /// <summary>
        ///     메모리 뷰 커맨드
        /// </summary>
        public RelayCommand MemoryViewCommand
        {
            get
            {
                return _memoryViewCommand ?? (_memoryViewCommand = new RelayCommand(obj =>
                {
                    if (_memoryViewPopup == null)
                    {
                        _memoryViewPopup = new Popup
                        {
                            Child = new MemoryViewer(),
                        };
                    }
                    var ele = FocusManager.GetFocusedElement() as FrameworkElement;
                    if (ele != null)
                    {
                        GeneralTransform transForm = ele.TransformToVisual(null);
                        Point point = transForm.TransformPoint(new Point());
                        _memoryViewPopup.VerticalOffset = point.Y + ele.ActualHeight;
                        _memoryViewPopup.HorizontalOffset = point.X;
                    }

                    _memoryViewPopup.IsOpen = !_memoryViewPopup.IsOpen;
                }));
            }
        }

        /// <summary>
        ///     팝업 출력 커맨드
        /// </summary>
        public RelayCommand OpenPopupCommand
        {
            get
            {
                return _openPopupCommand ?? (_openPopupCommand = new RelayCommand(obj =>
                {
                    var popup = new Popup
                    {
                        IsLightDismissEnabled = true,
                    };

                    var child = new TestPopup(popup) {Width = 300, Height = 200};
                    popup.Child = child;

                    ViewModelLocator.MemoryViewer.AddObject(popup);
                    popup.Closed += (s, e) =>
                    {
                        //child.ParentPopup = null;
                        //popup.Child = null;
                        //popup = null;
                    };
                    popup.IsOpen = true;
                }));
            }
        }

        /// <summary>
        ///     커맨드 활성/비활성 확인
        /// </summary>
        private void CheckCommands()
        {
            BitlyAuthCommand.RaiseCanExecuteChanged();
            GetShortenUrlCommand.RaiseCanExecuteChanged();
        }


        protected override void OnNavigated(NavigationEventArgs e)
        {
            //네비게이션이 완료 된 후 실행되는 부분, 지금은 디자인 타임과 동일한 데이터를 보여준다.
            ClassList = DesignDatas.GetClassList();
            People = DesignDatas.GetPeople();
        }
    }
}