﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using Windows.ApplicationModel;
using Windows.UI.Xaml;
using UniversalSample.Commons;
using UniversalSample.Models;

namespace UniversalSample.ViewModels
{
    /// <summary>
    /// MemoryViewerViewModel
    /// </summary>
    public class MemoryViewerVM : BindableBase
    {
        private ICollection<MemoryItemM> _memoryItems;

        /// <summary>
        ///     기본생성자
        /// </summary>
        public MemoryViewerVM()
        {
            MemoryItems = new ObservableCollection<MemoryItemM>();

            if (DesignMode.DesignModeEnabled)
            {
                MemoryItems.Add(new MemoryItemM("Test1"));
                MemoryItems.Add(new MemoryItemM("Test2"));
                MemoryItems.Add(new MemoryItemM("Test3"));
                MemoryItems.Add(new MemoryItemM("Test4"));
                MemoryItems.Add(new MemoryItemM("Test5"));
            }
            else
            {
#if DEBUG
                var dt = new DispatcherTimer {Interval = TimeSpan.FromSeconds(5)};
                dt.Tick += (s, e) => CheckMemoryItems();
                dt.Start();
#endif
            }
        }

        /// <summary>
        /// MemoryItems, 인스턴스된 객체 목록
        /// </summary>
        public ICollection<MemoryItemM> MemoryItems
        {
            get { return _memoryItems; }
            private set { SetProperty(ref _memoryItems, value); }
        }

        /// <summary>
        /// 메모리 아이템 체크, 5초에 한번씩 메모리를 확인해서 제거된 아이템을 삭제한다.
        /// </summary>
        private void CheckMemoryItems()
        {
            //MemoryItemM자체를 체크
            List<MemoryItemM> deads = MemoryItems.Where(p => p.IsAliveWeakReferences().Equals(false)).ToList();
            if (deads.Any())
            {
                foreach (MemoryItemM dead in deads)
                {
                    MemoryItems.Remove(dead);
                }
            }
        }

        /// <summary>
        ///     Object 추가
        /// </summary>
        /// <param name="obj"></param>
        public void AddObject(object obj)
        {
            CheckMemoryItems();

            //동일한 object가 이미 존재하는 경우
            MemoryItemM existItem = MemoryItems.FirstOrDefault(p => p.IsExist(obj));
            if (existItem != null) return;

            //동일한 이름의 MemoryItemM이 존재하는 경우
            MemoryItemM existItemName = MemoryItems.FirstOrDefault(p => p.ClassName.Equals(obj.GetType().FullName));
            if (existItemName != null)
            {
                existItemName.AddObject(obj);
            }
            else
            {
                MemoryItems.Add(new MemoryItemM(obj));
            }

            CheckMemoryItems();
        }
    }
}